#!/usr/bin/perl
#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCli.pl
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

package ClusterCli;

#**********************************************************************
#	Use Statements
#**********************************************************************
# Location of the Perl scripts
use constant BINDIR => "/opt/novell/ncs/bin";
use lib BINDIR;

# Location of the Snapins
use constant SNAPINPATH => "Snapins";
use lib $^O eq "linux" ? BINDIR . "/" . SNAPINPATH : SNAPINPATH;

use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			LoadCliSnapins
# Description:	Loads the Cluster CLI modules
# Parameters:	None.
# Return:		Hash of modules.  Hash format is
#						<command string> = <Object ref>
# Notes:
#**********************************************************************
sub LoadCliSnapins()
{
	my %snapins;
	my $ref;
	my $dirhandle;
	my @modules;
	my $mod;
	my $path;

	if ($^O eq "linux")
	{
		$path = BINDIR . "/" . SNAPINPATH;
	}
	else
	{
		$path = SNAPINPATH;
	} # if ($^O eq "linux")

	# Read the list of modules
	opendir($dirhandle, $path);
	@modules = grep(/^ClusterCliSnapin_\w*\.pm$/, readdir($dirhandle));
	closedir($dirhandle);

	# Load the modules if they are of the right type
	while ($mod = shift(@modules))
	{
		# trap any exceptions
		eval
		{
			require $mod;

			$mod = substr($mod, 0, -3);
			$ref = new $mod();
			if ($ref->isa("ClusterCliSnapinInterface"))
			{
				$snapins{lc($ref->Param())} = $ref;
			}
			else
			{
				console_print("WARNING: $mod is not a valid Cluster CLI snapin (1).\n");
			} # if ($ref->isa("ClusterCliSnapinInterface"))
		}; # eval
		if ($@)
		{
			console_print("WARNING: $mod is not a valid Cluster CLI snapin (2).\n");
		} # if ($@)
	} # while ($mod = shift(@modules))

	return(%snapins);

} # LoadCliSnapins


#**********************************************************************
# Name:			Main program entry point
# Description:	
# Parameters:	
# Return:		0     - Success
#				1     - Unknown error
#				2     - Snapin failure during Execute() method
#				3     - Snapin failure during Help() method
#				255   - Snapin does not exist
# Notes:		The return value should never be negative.  That way,
# 				NCSCON can determine if plose() fails.
#**********************************************************************

my $retVal = 1;

if (scalar(@ARGV))
{
    my $command = lc(shift(@ARGV));
	my %snapins = LoadCliSnapins();

	if ($command eq "help")
	{
		$command = shift(@ARGV);

		if (!defined($command) || "" eq $command)
		{
			my @verbs;

			@verbs = sort(keys(%snapins));

			console_print("\nCLUSTER HELP:\n");
			foreach $command (@verbs)
			{
				console_print("  $command\n");
			} # foreach $command (@verbs)
			console_print("\n");
			$retVal = 0;
		}
		else
		{
			$command = lc($command);
			if (exists($snapins{$command}))
			{
				$retVal = $snapins{$command}->Help(@ARGV) ? 0 : 3;
			} # if (%snapins{$command})
	
			# Don't output anything if the snapin does not exist
		} # if (!defined($command) || "" eq $command)
	}
	elsif (exists($snapins{$command}))
	{
		eval
		{
			$retVal = $snapins{$command}->Execute(@ARGV) ? 0 : 2;
		}; # eval
		if ($@)
		{
			console_print("\nThere was a problem executing command \"$command\".\n");
			console_print("Please try again.  If the problem persists, please\n");
			console_print("contact Novell Technical Support.\n");
		} # if ($@)
	}
	else
	{
		if (is_unknown_snapin_warning_displayed())
		{
            console_print("Unknown command: $command.\nTo see a list of cluster commands type \"cluster help\".\n");
		} # if (is_unknown_snapin_warning_displayed())

		$retVal = 0xff;
	}
} # if (scalar(@ARGV))

exit($retVal);

