#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapinInterface.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

package ClusterCliSnapinInterface;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use Fcntl;
use IO::Seekable;
use ClusterCliUtils;
use XML::LibXML;
require Exporter;
our @ISA = ("Exporter");
our @EXPORT = ("Param", "Execute", "Help");
our @EXPORT_OK = ("ParseBccResults");
use constant path_to_cmd => "/Novell/Cluster/";
#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("Invalid");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	return(0);

} # Execute

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("No help available.\n");
	return(0);

} # Help

#**********************************************************************
# Name:			GetHost
# Description:	Returns the host/volume where the VFS files are located.
#					The user will be prompted for a server name if necessary.
# Parameters:	None.
# Return:		Host and volume where VFS files reside.
# Notes:		*** Derived classes should not override this method ***
#**********************************************************************
sub GetHost()
{
	my ($self) = @_;
	my $host;

	if (!is_remote())
	{
		$host = !is_netware() ? "/admin" : "_ADMIN:";
	}
	else
	{
		$host = get_conf_option("host", "");

		if (length($host))
		{
			if ($host eq "localhost" || $host eq "127.0.0.1")
			{
				$host = !is_netware() ? "/admin" : "_ADMIN:";
			}
			else
			{
				$host = !is_netware() ? "//$host/admin" : "//$host/_admin";
			} # if ($host eq "localhost" || $host eq "127.0.0.1")
		}
		else
		{
			# conf file did not contain a host.  Query the user for the
			# remote server to try.
			console_print("Enter the name or IP address of the cluster to connect to: ");
			$host = <STDIN>;			# TODO: One day this should be changed to console_getstring()
			chomp($host);
		
			# The UNC path will allow the call to sysopen to attach to the remote
			# host if an authenticated connnection already exists.  If there is not
			# an authenticated connection then sysopen will fail.  Therefore, the
			# user must have an authenticated connection to the server for the UNC
			# path to work.
			$host = !is_netware() ? "//$host/admin" : "//$host/_admin";
		} # if (length($host))
	} # if (!is_remote())

	return($host);

} # GetHost

#**********************************************************************
# Name:			OpenVfsFile
# Description:	Opens a Virtual File System file for read/write access
# Parameters:	Host where VFS file resides.
#					File to open, including the path
# Return:		File handle of opened file
# Notes:			*** Derived classes should not override this method ***
#**********************************************************************
sub OpenVfsFile($$)
{
	# $_[0] is the self ref
	my $virtfile;
	my $filename = $_[1] . $_[2];

	# TODO: If the file is aready opened (e.g. in NetWare Edit.nlm), this call
	# will fail.  Need to gracefully handle this.

	if (!sysopen($virtfile, $filename, O_RDWR))
	{
		if (is_remote())
		{
			# If we are communicating with a remote cluster, we cannot
			# determine the remote platform. In this case we need to
			# try again with a converted path.
			$filename = convert_admin_path($filename);
			if ($filename && sysopen($virtfile, $filename, O_RDWR))
			{
				return($virtfile);
			} # if ($filename && sysopen($virtfile, $filename, O_RDWR))
		} # if (is_remote())
			
		console_print("Error communicating with cluster ($filename)\n");
		console_print("Is Novell Cluster Services running?\n");
		die("\n");		# Don't let die output a message
	} # if (!sysopen($virtfile, $filename, O_RDWR))

	return($virtfile);

} # OpenVfsFile

#**********************************************************************
# Name:			CloseVfsFile
# Description:	Closes an open Virtual File System file
# Parameters:	File handle to close
# Return:		true if close was successfull.  false otherwise.
# Notes:			*** Derived classes should not override this method ***
#**********************************************************************
sub CloseVfsFile(*)
{
	# $_[0] is the self ref
	close($_[1]);

} # CloseVfsFile

#**********************************************************************
# Name:			ReadVfsFile
# Description:	Reads data from a Virtual File System file
# Parameters:	File handle to use
#					Buffer to read into
# Return:		Number of bytes read
# Notes:			*** Derived classes should not override this method ***
#**********************************************************************
sub ReadVfsFile(*$)
{
	my $buf;

	# $_[0] is the self ref
	$_[2] = "";
	sysseek($_[1], 0, SEEK_SET);
	while(sysread($_[1], $buf, 1024))
	{
		$_[2] .= $buf;

		# Don't go into an infinite loop if VFS returns an error
		if ($buf =~ /<writeResult>[\s]*<error>.*?<\/error>[\s]*<\/writeResult>/s)
		{
			last;
		} # if ($buf =~ /<writeResult>[\s]*<error>.*?<\/error>[\s]*<\/writeResult>/s)
	} # while(sysread($_[1], $buf, 1024))

	return(length($_[2]));

} # ReadVfsFile

#**********************************************************************
# Name:			WriteVfsFile
# Description:	Writes data to a Virtual File System file
# Parameters:	File handle to use
#					Buffer to write
# Return:		Number of bytes written, or undef on error
# Notes:			*** Derived classes should not override this method ***
#**********************************************************************
sub WriteVfsFile(*$)
{
	# $_[0] is the self ref
	sysseek($_[1], 0, SEEK_SET);
	syswrite($_[1], $_[2], length($_[2]));

} # WriteVfsFile

#**********************************************************************
# Name:		GetInfoFromCMD
# Description:	returns the response for CMD request
# Parameters:	CMD file name, tag name, resource name
# Returns:	response of CMD request in xml form
# Notes:	*** Derived classes should not override this method ***
#**********************************************************************
sub GetInfoFromCMD(*$)
{
	my ($self,$fileName,$getReq ,$arr_ref, $resName) = @_;
	my @typeOfRequest = @{ $arr_ref };
	my $xml = $self->ClusterConfigXmlRequest(\@typeOfRequest,"$resName","$getReq");
	my $vfsfile = $self->cmdFile("$fileName");
	my $host = $self->GetHost();
	my $fh = $self->OpenVfsFile($host, $vfsfile);
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);
	my $data = do { local $/; $xml };
	my $doc = XML::LibXML->load_xml(string => $data );
	return($doc);
}

#**********************************************************************
# Name:		ClusterConfigXmlRequest
# Description:	returns the resqest for ClusterConfig CMD
# Parameters:	tag name, resource name
# Returns:	resquest for ClusterConfig CMD
# Notes:	*** Derived classes should not override this method ***
#**********************************************************************

sub ClusterConfigXmlRequest(*$)
{
	my ($self , $arr_ref , $resName,$getReq) = @_;
	my @typeOfRequest = @{ $arr_ref };
	my $xml;
        $xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	if("$getReq" eq "getResourceInfo")
	{
		$xml .= "<ncsRequest><clusterConfig><getResourceInfo><typeOfInfo>";
		foreach my $req (@typeOfRequest)
		{
			$xml .= "<$req/>";
		}
		$xml .= "</typeOfInfo><name>$resName</name></getResourceInfo></clusterConfig></ncsRequest>";
	}
	elsif("$getReq" eq "getClusterInfo")
	{
		$xml .= "<ncsRequest><clusterConfig><getClusterInfo><typeOfInfo>";
		foreach my $req (@typeOfRequest)
		{
			$xml .= "<$req/>";
		}
		$xml .= "</typeOfInfo></getClusterInfo></clusterConfig></ncsRequest>";
	}
	return $xml;
}

#**********************************************************************
# Name:		cmdFile
# Description:	returns the path to CMD file
# Parameters:	CMD file name
# Returns:	path to the CMD file.
# Notes:	*** Derived classes should not override this method ***
#**********************************************************************

sub cmdFile(*)
{
	my $self = shift;
        return(path_to_cmd."$_[0]");
}

#**********************************************************************
# Name:		GetCurrentNode
# Description:	returns the current node
# Parameters:
# Returns:	current node name
# Notes:	*** Derived classes should not override this method ***
#**********************************************************************

sub GetCurrentNode()
{
	my $self = shift;
	my $node;
	my $xml2;
        my $host2 = $self->GetHost();
        my $fh2 = $self->OpenVfsFile($host2, $self->cmdFile("NodeState.xml"));
        $self->ReadVfsFile($fh2,$xml2);
        my $data = do { local $/; $xml2 };
        my $doc = XML::LibXML->load_xml(string => $data );
        foreach my $pnode ($doc->findnodes('//thisNode'))
        {
             $node = $pnode->to_literal();
        }
	return("$node");
}

#**********************************************************************
# Name:			ParseBccResults
# Description:	Parses the results from a BCC command
# Parameters:	Buffer to parse
#					Hash to place results in.
#						BccReplyCode - BCC error code
#						BccReplyString - BCC error string
#					Base reply tag
#					Vert tag
# Return:		true on success.  false otherwise.
# Notes:			*** Derived classes should not override this method ***
#**********************************************************************
sub ParseBccResults($$$$)
{
	my ($self, $xml, $output, $baseTag, $verbTag) = @_;

	$output->{"BccReplyCode"} = "1000";
	$output->{"BccReplyString"} = "Error parsing results";

	if ($xml =~/<bccReply>(.*?)<\/bccReply>/s)
	{
		$xml = $1;
		if ($xml =~/<$baseTag>(.*?)<\/$baseTag>/s)
		{
         $xml = $1;
			if ($xml =~/<$verbTag>(.*?)<\/$verbTag>/s)
			{
				if ($xml =~/<result value=\"([\d]+)\">(.*?)<\/result>/s)
				{
					$output->{"BccReplyCode"} = $1;
					$xml = $2;

					if ($xml =~/<description>(.*?)<\/description>/s)
					{
						$output->{"BccReplyString"} = $1;
						return(1);
					}
				}
			}
		}
	}
	# Check for IPC error
	elsif ($xml =~ m/<result value=\"([\d]+)\"><description>(.*?)<\/description><\/result>/s)
	{
		# Check for an IPC timeout error
		if ("1029" eq $1)
		{
			console_print("A timeout occured while waiting for a result.\n");
			console_print("BCC is currently busy performing another operation.\n");
		}
		else
		{
			console_print("An internal error ($1) occured while servicing this request.\n");
		} # if ("1029" eq $1)

		# Print the description iff available
		if ("" ne $2)
		{
			console_print("$2\n");
		} # if ("" ne $2)

		# The IPC is down, so abort the CLI
		throw();
	} # if ($xml =~/<bccReply>(.*?)<\/bccReply>/s)

	return(0);

} # ParseBccResults

#**********************************************************************
# Name:			ShowResourceList
# Description:	Displays the list of resources
# Parameters:	string	- Host to query data from
#					bool		- true to show header, false to not show it.
#					string	- Resource to show.  null to show all.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ShowResourceList($$$)
{
	my ($self, $host, $showHeader, $resource) = @_;
	my $fh;
	my $xml;
	my $vfsfile	= "/Novell/Cluster/NodeState.xml";
	my $retVal;

	# Output the standard NCS information
	$fh = $self->OpenVfsFile($host, $vfsfile);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	$xml =~ /<ncsReply>[\s]*<cluster>(.*?)<\/cluster>[\s]*<\/ncsReply>/s;
	$xml = $1;

	#TODO: Need to handle error conditions better
	if (!$xml)
	{
		console_print("This node is not a member of a cluster\n");
		return(1);
	} # if ($xml)

	if ($self->IsBCCLoaded($host))
	{
		$retVal = $self->ShowBCCResourceList($host, $showHeader, $resource);
                # if the ShowBCCResourceList fails, at least show the NCSResourceList
                if($retVal ne 1)
                {
                   $retVal = $self->ShowNCSResourceList($host, $showHeader, $resource);
                }
	}
	else
	{
		$retVal = $self->ShowNCSResourceList($host, $showHeader, $resource);
	} # if ($self->IsBCCLoaded($host))

	return($retVal);

} # ShowResourceList

#**********************************************************************
# Name:			ShowNCSResourceList
# Description:	Displays the list of resources for a cluster without BCC
# Parameters:	string	- Host to query data from
#					bool		- true to show header, false to not show it.
#					string	- Resource to show.  null to show all.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ShowNCSResourceList($$$)
{
	my ($self, $host, $showHeader, $resource) = @_;
	my $fh;
	my $xml;
	my @reslist;
	my $res;
	my $state;
	my $node;
	my $lives;

	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	$xml .= "<ncsRequest><clusterManage><listClusterResources/>";
	$xml .= "</clusterManage></ncsRequest>";

	log_request($xml);

	# Query the admin volume
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/ClusterManage.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	$xml =~ /<ncsReply>[\s]*<clusterManage>[\s]*<listClusterResources>(.*?)<\/listClusterResources>[\s]*<\/clusterManage>[\s]*<\/ncsReply>/s;
	$xml = $1;

	if (defined($xml))
	{
		@reslist = $self->SortByElement("name", "Master_IP_Address_Resource", $xml =~ /<resourceInfo>(.*?)<\/resourceInfo>/gs);
	
		if ($resource)
		{
			my $revision;

			foreach $xml (@reslist)
			{
				if ($xml =~ /<name>(.*?)<\/name>/s)
				{
					$res = $1;
				}
	
				if ($res && lc($res) eq lc($resource))
				{
					if ($xml =~ /<state>(.*?)<\/state>/s)
					{
						$state = $1;
					}
					else
					{
						$state = "<unknown>";
					}
					if ($xml =~ /<location>(.*?)<\/location>/s)
					{
						$node = $1;
					}
					if ($xml =~ /<incarnation>(.*?)<\/incarnation>/s)
					{
						$lives = $1;
					}
					else
					{
						$lives = "<unknown>";
					}
					if ($xml =~ /<revision>(.*?)<\/revision>/s)
					{
						$revision = $1;
					}
					else
					{
						$revision = "0";
					}

					# The response doesn't always include a node (e.g. if the state is offline).
					if ($node)
					{
						$state = "$state on $node";
					} # if ($node)
	
					console_print("\nStatus for Resource: $res\n");
					console_print("$state  Lives: $lives\n");
					console_print("Revision: $revision\n");
	
					last;
				} #if ($res && lc($res) eq lc($resource))
			} # foreach $xml (@reslist)
		}
		else
		{
			my $format = "@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<< @<<<<<<<<<<<<<<<< @>>>>\n";
			my $max_name_len = 43;
			my $max_state_len = 9;
			my $max_loc_len = 16;
			my $len = 0;

			foreach $xml (@reslist)
			{
				if ($xml =~ /<name>(.*?)<\/name>/s)
				{
					$res = $1;
				}
				else
				{
					$res = "<unknown>";
				}
				if( $max_name_len < ($len = length( $res )))
				{
					$max_name_len = $len;
				}

				if ($xml =~ /<state>(.*?)<\/state>/s)
				{
					$state = $1;
				}
				else
				{
					$state = "<unknown>";
				}
				if( $max_state_len < ($len = length( $state )))
				{
					$max_state_len = $len;
				}

				if ($xml =~ /<location>(.*?)<\/location>/s)
				{
					$node = $1;
				}
				else
				{
					$node = " ";		# Offline does not have a location
				}
				if( $max_loc_len < ($len = length( $node )))
				{
					$max_loc_len = $len;
				}
			} # foreach $xml (@reslist)

			if( 43 < $max_name_len || 9 < $max_state_len || 16 < $max_loc_len )
			{
				$format="@" . "<" x $max_name_len . " @" . "<" x $max_state_len . " @" . "<" x $max_loc_len . " @>>>>\n";
			}

			# Clear the format output accumulator
			$^A = "";
		
			if ($showHeader)
			{
				$res		= "Name";
				$state		= "State";
				$node  		= "Node";
				$lives		= "Lives";
				formline($format, ($res, $state, $node, $lives));
			} # if ($showHeader)
		
			foreach $xml (@reslist)
			{
				if ($xml =~ /<name>(.*?)<\/name>/s)
				{
					$res = $1;
				}
				else
				{
					$res = "<unknown>";
				}
				if ($xml =~ /<state>(.*?)<\/state>/s)
				{
					$state = $1;
				}
				else
				{
					$state = "<unknown>";
				}
				if ($xml =~ /<location>(.*?)<\/location>/s)
				{
					$node = $1;
				}
				else
				{
					$node = " ";		# Offline does not have a location
				}
				if ($xml =~ /<incarnation>(.*?)<\/incarnation>/s)
				{
					$lives = $1;
				}
				else
				{
					$lives = "<unknown>";
				}
			 formline($format, ($res, $state, $node, $lives));
			} # foreach $xml (@reslist)
			console_print($^A);
			$^A = "";
		} # if ($resource)
	}
	else
	{
		console_print("There was an error retrieving the status from NCS.  If the error\n");
		console_print("persists, please contact Novell Technical Support\n");		
	} #if (defined($xml))
	
	return(1);

} # ShowNCSResourceList

#**********************************************************************
# Name:			ShowBCCResourceList
# Description:	Displays the list of resources for a cluster with BCC
# Parameters:	string	- Host to query data from
#					bool		- true to show header, false to not show it.
#					string	- Resource to show.  null to show all.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ShowBCCResourceList($$$)
{
	my ($self, $host, $showHeader, $resource) = @_;
	my %params;
	my $fh;
	my $xml;
	my @reslist;
	my $res;
	my $ncsstate;
	my $bccstate;
	my $location;
	my $lives;
	my $bIsBCCResource;

	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>" .
		"<bccRequest><bccManage><getResourceStatus>" .
		"<cluster>.</cluster>" .				# . = current cluster.  Could change this to work with remote clusters...
		"</getResourceStatus></bccManage></bccRequest>";

	log_request($xml);

	# Query the admin volume
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Manage.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

        # Verify that we were actually able to get a response from BCC
        # If no response was returned then we need to return the error here so that
        # we can run the NCSGetResourceStatus option
        if ($xml =~ m/<result value=\"([\d]+)\"><description>(.*?)<\/description><\/result>/s)
        {
           return $1;
        }

	$self->ParseBccResults($xml, \%params, "bccManage", "getResourceStatus");
	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	$xml =~ /<bccReply>[\s]*<bccManage>[\s]*<getResourceStatus>(.*?)<\/getResourceStatus>[\s]*<\/bccManage>[\s]*<\/bccReply>/s;

	if ((0 == $params{"BccReplyCode"}) and defined($1))
	{
		$xml = $1;

		@reslist = $self->SortByElement("name", "Master_IP_Address_Resource", $xml =~ /<statusInfo>(.*?)<\/statusInfo>/gs);
	
		if ($resource)
		{
			my $revision;

			foreach $xml (@reslist)
			{
				$bIsBCCResource = 0;

				if ($xml =~ /<resourceType>(.*?)<\/resourceType>/s)
				{
					if ($1 eq "<bccResource/>" || $1 eq "<bccPool/>")
					{
						$bIsBCCResource = 1;
					} # if ($1 eq "bccResource" || $1 eq "bccPool")
				} # if ($xml =~ /<resourceType>(.*?)<\/resourceType>/s)

				if ($xml =~ /<name>(.*?)<\/name>/s)
				{
					$res = $1;
				}
	
				if ($res && lc($res) eq lc($resource))
				{
					if ($xml =~ /<state>(.*?)<\/state>/s)
					{
						$ncsstate = $1;
					}
					else
					{
						$ncsstate = "<unknown>";
					}
					if ($xml =~ /<location>(.*?)<\/location>/s)
					{
						$location = $1;
					}
					if ($xml =~ /<incarnation>(.*?)<\/incarnation>/s)
					{
						$lives = $1;
					}
					else
					{
						$lives = "<unknown>";
					}
					if ($xml =~ /<revision>(.*?)<\/revision>/s)
					{
						$revision = $1;
					}
					else
					{
						$revision = "0";
					}

					if ($bIsBCCResource)
					{
						if ($ncsstate eq "Secondary")
						{
							if ($location)
							{
								$ncsstate = "Offline (Primary on peer $location)";
							}
							else
							{
								$ncsstate = "Offline";
							}
							
							$bccstate = "Secondary";
						}
						else
						{
							# The response doesn't always include a node (e.g. if the state is offline).
							if ($location)
							{
								$ncsstate = "$ncsstate on $location";
							} # if ($location)
		
							$bccstate = "Primary";
						} # if ($ncsstate eq "Secondary")
					}
					else
					{
						# The response doesn't always include a node (e.g. if the state is offline).
						if ($location)
						{
							$ncsstate = "$ncsstate on $location";
						} # if ($location)

						$bccstate = "---";
					} # if ($bIsBCCResource)

					console_print("\nStatus for Resource: $res\n");
					console_print("$ncsstate  Lives: $lives\n");
					console_print("Revision: $revision  BCC State: $bccstate\n");

					last;
				} #if ($res && lc($res) eq lc($resource))
			} # foreach $xml (@reslist)
		}
		else
		{
			my $format = "@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< @<<<<<<<<< @<<<<<<<<< @<<<<<<<<<<<<<<<< @>>>>\n";
			my $max_name_len = 32;
			my $max_state_len = 9;
			my $max_loc_len = 16;
			my $len = 0;

			foreach $xml (@reslist)
			{
				$bIsBCCResource = 0;

				if ($xml =~ /<resourceType>(.*?)<\/resourceType>/s)
				{
					if ($1 eq "<bccResource/>" || $1 eq "<bccPool/>")
					{
						$bIsBCCResource = 1;
					} # if ($1 eq "bccResource" || $1 eq "bccPool")
				} # if ($xml =~ /<resourceType>(.*?)<\/resourceType>/s)

				if ($xml =~ /<name>(.*?)<\/name>/s)
				{
					$res = $1;
				}
				else
				{
					$res = "<unknown>";
				}
				if( $max_name_len < ($len = length( $res )))
				{
					$max_name_len = $len;
				}

				if ($xml =~ /<state>(.*?)<\/state>/s)
				{
					$ncsstate = $1;
				}
				else
				{
					$ncsstate = "<unknown>";
				}
				if( $max_state_len < ($len = length( $ncsstate )))
				{
					$max_state_len = $len;
				}

				if ($xml =~ /<location>(.*?)<\/location>/s)
				{
					$location = $1;
				}
				else
				{
					$location = " ";		# Offline does not have a location
				}
				if( $max_loc_len < ($len = length( $location )))
				{
					$max_loc_len = $len;
				}

				if ($bIsBCCResource)
				{
					if ($ncsstate eq "Secondary")
					{
						$ncsstate = "Offline";
						$bccstate = "Secondary";
					}
					else
					{
						$bccstate = "Primary";
					} # if ($ncsstate eq "Secondary")
				}
				else
				{
					$bccstate = "---";	# Not a BCC resource.  Don't show a BCC state
				} # if ($bIsBCCResource)
				if( $max_state_len < ($len = length( $bccstate )))
				{
					$max_state_len = $len;
				}
			}

			if( 32 < $max_name_len || 9 < $max_state_len || 16 < $max_loc_len )
			{
				$format="@" . "<" x $max_name_len . " @" . "<" x $max_state_len . " @" . "<" x $max_state_len . " @" . "<" x $max_loc_len . " @>>>>\n";
			}

			# Clear the format output accumulator
			$^A = "";
		
			if ($showHeader)
			{
				$res		= "Name";
				$ncsstate	= "State";
				$bccstate	= "BCC State";
				$location	= "Location";
				$lives		= "Lives";
				formline($format, ($res, $ncsstate, $bccstate, $location, $lives));;
			} # if ($showHeader)
		
			foreach $xml (@reslist)
			{
				$bIsBCCResource = 0;

				if ($xml =~ /<resourceType>(.*?)<\/resourceType>/s)
				{
					if ($1 eq "<bccResource/>" || $1 eq "<bccPool/>")
					{
						$bIsBCCResource = 1;
					} # if ($1 eq "bccResource" || $1 eq "bccPool")
				} # if ($xml =~ /<resourceType>(.*?)<\/resourceType>/s)

				if ($xml =~ /<name>(.*?)<\/name>/s)
				{
					$res = $1;
				}
				else
				{
					$res = "<unknown>";
				}
				if ($xml =~ /<state>(.*?)<\/state>/s)
				{
					$ncsstate = $1;
				}
				else
				{
					$ncsstate = "<unknown>";
				}
				if ($xml =~ /<location>(.*?)<\/location>/s)
				{
					$location = $1;
				}
				else
				{
					$location = " ";		# Offline does not have a location
				}
				if ($xml =~ /<incarnation>(.*?)<\/incarnation>/s)
				{
					$lives = $1;
				}
				else
				{
					$lives = "<unknown>";
				}

				if ($bIsBCCResource)
				{
					if ($ncsstate eq "Secondary")
					{
						$ncsstate = "Offline";
						$bccstate = "Secondary";
					}
					else
					{
						$bccstate = "Primary";
					} # if ($ncsstate eq "Secondary")
				}
				else
				{
					$bccstate = "---";	# Not a BCC resource.  Don't show a BCC state
				} # if ($bIsBCCResource)
	
			 formline($format, ($res, $ncsstate, $bccstate, $location, $lives));
			} # foreach $xml (@reslist)
	
			console_print($^A);
			$^A = "";
		} # if ($resource)
	}
	else
	{
		console_print("$params{\"BccReplyString\"} ($params{\"BccReplyCode\"})\n");
		return(0);
	} #if (defined($xml))
	
	return(1);

} # ShowBCCResourceList

#**********************************************************************
# Name:			ShowBccPeers
# Description:	Displays the list of BCC peer clusters on STDOUT
# Parameters:	string	- Host to query data from
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ShowBccPeers($)
{
	my ($self, $host) = @_;
	my $vfsfile = "/Novell/Cluster/BCC_Config.cmd";
	my $fh;
	my $xml;
	my @templist;
	my $temp;

	# Output the BCC peer information
	if ($self->IsBCCLoaded($host))
	{
		$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>" .
			"<bccRequest><bccConfig><getClusterInfo/></bccConfig></bccRequest>";

		log_request($xml);

		$fh = $self->OpenVfsFile($host, $vfsfile);
		$self->WriteVfsFile($fh, $xml);
		$self->ReadVfsFile($fh, $xml);
		$self->CloseVfsFile($fh);

		log_response($xml);

		if ($xml =~ /<enabled\/>/ &&
			$xml =~ /<bccReply>[\s]*<bccConfig>[\s]*<getClusterInfo>.+<peers>(.*?)<\/peers>.*?<\/getClusterInfo>[\s]*<\/bccConfig>[\s]*<\/bccReply>/s)
		{
			$xml = $1;
			@templist = $xml =~ /<peer>.*?<cluster>(.*?)<\/cluster>.*?<\/peer>/gs;
		}
		else
		{
			# BCC is disabled or no peer data was returned. Most likely, this node is not a
			# member of the cluster or BCC has not refreshed yet
			console_print("\n");
			return(1);
		}
		
		console_print("BCC peer clusters [ ");
		for (my $idx = 0; $idx < @templist; $idx++)
		{
			if ($idx != 0)
			{
				$temp = ", $templist[$idx]";
			}
			else
			{
				$temp = $templist[$idx];
			} # if ($idx != 0)

			console_print($temp);
		} # foreach $node (@nodelist)
		console_print(" ]\n");
	} # if ($self->IsBCCLoaded($host))

	console_print("\n");

	return(1);

} # ShowBccPeers

#**********************************************************************
# Name:			GetClusterName
# Description:	Return the local cluster name
# Parameters:	None
# Return:		Cluster name, or empty string on error
# Notes:
#**********************************************************************
sub GetClusterName($)
{
	my ($self, $host) = @_;
	my $fh;
	my $xml;
	my $cluster;
	my $vfsfile	= "/Novell/Cluster/NodeState.xml";

	# Output the standard NCS information
	$fh = $self->OpenVfsFile($host, $vfsfile);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	$xml =~ /<ncsReply>[\s]*<cluster>(.*?)<\/cluster>[\s]*<\/ncsReply>/s;
	$xml = $1;

	if ($xml)
	{
		if ($xml =~ /<name>(.*?)<\/name>/s)
		{
			return($1);
		}
	}
	return("");
} # GetClusterName

#**********************************************************************
# Name:			ShowClusterView
# Description:	Does a "Cluster View" and displays the info
# Parameters:	string	- Host to query data from
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ShowClusterView($)
{
	my ($self, $host) = @_;
	my $fh;
	my $xml;
	my $cluster;
	my $current;
	my $epoch;
	my $master;
	my @templist;
	my $temp;
	my $vfsfile	= "/Novell/Cluster/NodeState.xml";

	# Output the standard NCS information
	$fh = $self->OpenVfsFile($host, $vfsfile);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	$xml =~ /<ncsReply>[\s]*<cluster>(.*?)<\/cluster>[\s]*<\/ncsReply>/s;
	$xml = $1;

	#TODO: Need to handle error conditions better
	if ($xml)
	{
		if ($xml =~ /<name>(.*?)<\/name>/s)
		{
			$cluster = $1;
		}
		else
		{
			$cluster = "<unknown>";
		}
		if ($xml =~ /<thisNode>(.*?)<\/thisNode>/s)
		{
			$current = $1;
		}
		else
		{
			$current = "<unknown>";
		}
		if ($xml =~ /<epoch>(.*?)<\/epoch>/s)
		{
			$epoch = $1;
		}
		else
		{
			$epoch = "<unknown>";
		}
		if ($xml =~ /<masterNode>(.*?)<\/masterNode>/s)
		{
			$master = $1;
		}
		else
		{
			$master = "<unknown>";
		}
	
		$xml =~ /<nodes>(.*?)<\/nodes>/s;
		$xml = $1;
		@templist = $xml =~ /<node>(.*?)<\/node>/gs;
	
		console_print("\n\tCluster $cluster\n");
		console_print("\tThis node $current [ epoch $epoch master node $master ]\n");
		console_print("\tCluster nodes [ ");
		for (my $idx = 0; $idx < @templist; $idx++)
		{
			if ($idx != 0)
			{
				$temp = ", $templist[$idx]";
			}
			else
			{
				$temp = $templist[$idx];
			} # if ($idx != 0)
	
			console_print($temp);
		} # for (my $idx = 0; $idx < @templist; $idx++)
		console_print(" ]\n\t");
	
		# Output the BCC peer information
		$self->ShowBccPeers($host);
	}
	else
	{
		console_print("This node is not a member of a cluster\n");
	} # if ($xml)
	
	return(1);

} # ShowClusterView

#**********************************************************************
# Name:			CheckClusterConnections
# Description:	Check that the current cluster's connections are okay.
# Parameters:	None
# Return:		1 if connections are OK, 0 otherwise.
# Notes:
#**********************************************************************
sub CheckClusterConnections()
{
	my $self = shift;
	my %params;
	my $host = $self->GetHost();
	my $fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Manage.cmd");
	my $xml =
		"<virtualIO>".
			"<datastream name=\"command\"/>".
		"</virtualIO>".
		"<bccRequest>".
			"<bccManage>".
				"<getConnections>".
					"<cluster>.</cluster>".
				"</getConnections>".
			"</bccManage>".
		"</bccRequest>";

	log_request($xml);
	$self->WriteVfsFile($fh, $xml);

	$self->ReadVfsFile($fh, $xml);
	log_response($xml);

	$self->CloseVfsFile($fh);

	$self->ParseBccResults($xml, \%params, "bccManage", "getConnections");
	if (0 != $params{"BccReplyCode"})
	{
		console_print("Error retrieving connections status.\n");
		console_print("$params{\"BccReplyString\"} ($params{\"BccReplyCode\"})\n");
		die();
	} # if (0 != $params{"BccReplyCode"})

	while ($xml =~ /<status>(.*)<\/status>/g)
	{
		if ($1 ne "okay")
		{
			return(0);
		}
	} # while ($xml =~ /<status>(.*)<\/status>/g)

	return(1);
} # CheckClusterConnections

#**********************************************************************
# Name:			IsBCCLoaded
# Description:	Determines if the BCC is loaded or not
# Parameters:	string	- Host
# Return:		bool	- 1 if bcc is loaded, 0 if not
# Notes:        --ncs passed in as a command line argument means to bypass bcc
#               Its not a published interface but used by the ncsSDK.
#               e.g. cluster resources --ncs 
#**********************************************************************
sub IsBCCLoaded($)
{
	my ($self, $host) = @_;
	my $filename = $host."/Novell/Cluster/BCC.xml";
	
	if (is_force_ncs_set())
	{
		return 0;
	} # if (is_force_ncs_set())

	if (stat($filename))
	{
		return(1);
	} # if (stat($filename))

	if (is_remote())
	{
		# If we are communicating with a remote cluster, we cannot
		# determine the remote platform. In this case we need to
		# try again with a converted path.
		$filename= convert_admin_path($filename);
		if ($filename && stat($filename))
		{
			return(1);
		} # if ($filename && stat($filename))
	} # if (is_remote())

	return(0);
} # IsBCCLoaded

#**********************************************************************
# Name:			IsMember
# Description:	Determines if the node is a member of the cluster or not
# Parameters:	String	- Host
# Return:		bool	- 1 if Node is a member.  0 otherwise.
# Notes:
#**********************************************************************
sub IsMember($)
{
	my ($self, $host) = @_;
	my $fh;
	my $xml;
	my $vfsfile	= "/Novell/Cluster/NodeState.xml";
	my $member = 0;

	# Read the node state
	$fh = $self->OpenVfsFile($host, $vfsfile);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	$xml =~ /<ncsReply>[\s]*<cluster>(.*?)<\/cluster>[\s]*<\/ncsReply>/s;
	$xml = $1;

	if ($xml)
	{
		$member = 1;
	} # if ($xml)

	return($member);

} # IsMember

#**********************************************************************
# Name:			SortByElement
# Description:	Sorts a list of XML strings by a given element
# Parameters:   string	- XML element to sort by
# 				string	- Optional element value to force to the front of the list
#				list	- List of XML strings to sort
# Return:		list	- The sorted list
# Notes:
#**********************************************************************
sub SortByElement($$@)
{
	my ($self, $element, $first, @list) = @_;

	return(sort(
		{
			my $ret;
			my $element1 = ($a =~ /<$element>(.*?)<\/$element>/s)[0];
			my $element2 = ($b =~ /<$element>(.*?)<\/$element>/s)[0];

			if (defined($first) && 0 != length($first))
			{
				if ($first eq $element1)
				{
					return(-1);
				}
				elsif ($first eq $element2)
				{
					return(1);
				} # if ($first eq $element1)
			} # if (defined($first) && 0 != length($first))

			return($element1 cmp $element2);
		}
		@list));

} # SortByElement

#**********************************************************************
# Name:			new
# Description:	Constructs the snapin object
# Parameters:	Name of the class being constructed
# Return:		reference to the object
# Notes:			*** Derived classes should not override this method ***
#**********************************************************************
sub new($)
{
	my $class = shift;
	my $self = { };
	bless($self, $class);

	return($self);

} # new

1;
#**********************************************************************
# Name:         ScanForNewDevices
# Description:  Issue the command to all nodes to scan for new devices
# Parameters:   None
# Return:       true on success.  false otherwise.
# Notes:                        
#**********************************************************************
sub ScanForNewDevices($)
{
	my $rc = 0;
	my $cmd = "EXEC /opt/novell/ncs/bin/device_scan.sh";
	my $file_handle;

	if( sysopen( $file_handle, "/proc/ncs/cluster", O_WRONLY ))
	{
		syswrite( $file_handle, $cmd, length( $cmd ));
		close( $file_handle );
		$rc = 1;
	}

	return($rc);
}

