#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliUtils.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

package ClusterCliUtils;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use Fcntl;
use IO::Seekable;
use Getopt::Long;

# Export our symbols
require Exporter;
our @ISA = ("Exporter");
our @EXPORT = qw(convert_admin_path is_prompt_set is_remote is_force_ncs_set is_unknown_snapin_warning_displayed is_netware console_print console_getstring console_getpassword get_conf_option log_request log_response);

#**********************************************************************
#	Global Data
#**********************************************************************
my $g_bRemote;
my $g_bDebug;
my $g_bForceNCS;
my $g_bNoUnknown;
my $g_bNoPrompt;
my $g_strConfFile;
my %g_Options;

#**********************************************************************
#	Functions
#**********************************************************************
#**********************************************************************
# Name:			INIT
# Description:	Runs before the program executes
# Notes:
#**********************************************************************
INIT
{
	my $path;

	$g_bRemote = 0;
	$g_bForceNCS = 0;

	# Read the options
	parse_command_options();

	# Read the conf file
	parse_conf_file();

	# Talking to a remote box? (eg running CLI on a windows host)
	my $host = get_conf_option("host", "");
	if (length($host))
	{
		if ($host ne "localhost" && $host ne "127.0.0.1")
		{
			$g_bRemote = 1;
		} # if ($host ne "localhost" && $host ne "127.0.0.1")
	}
	else
	{
		$path = !is_netware() ?
			"/admin/Novell/Cluster/NCS.xml" :
			"_ADMIN:/Novell/Cluster/NCS.xml";

		if (!stat($path))
		{
			$g_bRemote = 1;
		} # if (!stat($path))
	} # if (length($host))

	# Debugging?
	$g_bDebug = "true" eq get_conf_option("debug", "false");

} # INIT

#**********************************************************************
# Name:			convert_admin_path
# Description:	Convert an //host/admin/... style path to an
#				//host/_admin/... style path or viceversa.
# Parameters:	None
# Return:		the converted path, or 0 if the path was invalid.
# Notes:
#**********************************************************************
sub convert_admin_path($)
{
	$g_bRemote || die("Function called when it should not have been.\n".
		"Please make sure the cluster CLI is configured properly.");

	my $path = shift();

	if ($path =~ s/\/admin\//\/_admin\// || $path =~ s/\/_admin\//\/admin\//)
	{
		return($path);
	} # if ($path =~ s/\/admin\//\/_admin\// || $path =~ s/\/_admin\//\/admin\//)

	return(0);
} # convert_admin_path

#**********************************************************************
# Name:			is_remote
# Description:	Determines if the script is executing on the local server
#				or remotely.
# Parameters:	None
# Return:		true if remote.  false if on server.
# Notes:
#**********************************************************************
sub is_remote()
{
	return($g_bRemote);
} # is_remote

#**********************************************************************
# Name:			is_prompt_set
# Description:	Determines if the the user should be prompted for interaction.
# Parameters:	None
# Return:		true if the user should be prompted.  false otherwise.
# Notes:
#**********************************************************************
sub is_prompt_set()
{
	return(!$g_bNoPrompt);
} # is_prompt_set

#**********************************************************************
# Name:			is_force_ncs_set
# Description:	Determines if the NCS snapins should be forced, even
#				if BCC is loaded
# Parameters:	None
# Return:		true if NCS snapins should be forced.  false otherwise.
# Notes:
#**********************************************************************
sub is_force_ncs_set()
{
	return($g_bForceNCS);
} # is_force_ncs_set

#**********************************************************************
# Name:			is_unknown_snapin_warning_displayed
# Description:	Determines if the unknown snapin warning is displayed
#  Parameters:	None
# Return:		true if unknown snapin warning should be shown.
# 				false otherwise.
# Notes:
#**********************************************************************
sub is_unknown_snapin_warning_displayed()
{
	return(!$g_bNoUnknown);
} # is_unknown_snapin_warning_displayed

#**********************************************************************
# Name:			is_netware
# Description:	Determines if the CLI is running on NetWare or some other OS
#  Parameters:	None
# Return:		true if running on NetWare.  false otherwise.
# Notes:
#**********************************************************************
sub is_netware()
{
	my $host_type;

	$host_type = get_conf_option("host_type", "");
	if (0 != length($host_type))
	{
		return($host_type eq "NetWare");		
	} # if (0 != length($host_type))

	return($^O eq "NetWare");
} # is_netware

#**********************************************************************
# Name:			console_print
# Description:	Prints output to the screen the CLI is running in
# Parameters:	String to print
# Return:		true on success.  false otherwise
# Notes:
#**********************************************************************
sub console_print(@)
{
	print(@_);
	return(1);

} # console_print

#**********************************************************************
# Name:			console_getstring
# Description:	Retrieves a string from input on the console
# Parameters:	None.
# Return:		The string, or "" if none or error
# Notes:
#**********************************************************************
sub console_getstring()
{
	my $string = "";

	chomp($string = <STDIN>);

	return($string);

} # console_getstring

#**********************************************************************
# Name:			console_getpassword
# Description:	Prompts for and returns a password.
# Parameters:	None
# Return:		The password, or "" if none or error.
# Notes:		*** For security reasons, this can only execute on  ***
#				*** the server console                              ***
#**********************************************************************
sub console_getpassword()
{
	my $password = "";

	if (!is_remote())
	{
		system("stty -echo");
		$password = console_getstring();
		system("stty echo");
		console_print("\n");	# Need to echo a newline
	}
	else
	{
		# Remote passwords are not allowed
		console_print("Remote passwords are not allowed\n");
	} # if (!is_remote())

	return($password);

} # console_getpassword

#**********************************************************************
# Name:			parse_command_options
# Description:	Parses the command line options
# Parameters:	None
# Return:		None
# Notes:
#**********************************************************************
sub parse_command_options()
{
	my %opts = ('c=s' => \$g_strConfFile,		# -c ==> Specify the config file
				'ncs' => \$g_bForceNCS,			# --ncs ==> Force the NCS snapins to be used, even if BCC is present
				'nounknown' => \$g_bNoUnknown,	# --nounknown ==> Don't show the warning message for unknown snapins
				'noprompt' => \$g_bNoPrompt);	# --noprompt ==> Don't prompt the user for interaction.

	Getopt::Long::Configure("permute");
	Getopt::Long::Configure("pass_through");
	GetOptions(%opts);

} # parse_command_options

#**********************************************************************
# Name:			parse_conf_file
# Description:	Parses the conf file and saves the options in a hash
# Parameters:	None
# Return:		1 on success.  0 otherwise
# Notes:
#**********************************************************************
sub parse_conf_file()
{
	my $fh;
	my $file;
	my $line;
	my $idx;

	if (!defined($g_strConfFile))
	{
		foreach my $path (@INC)
		{
			$file = $path . "/clustercli.conf";

			if (stat($file))
			{
				last;				
			} # if (stat($file))			
		} # foreach my $path (@INC)
	}
	else
	{
		$file = $g_strConfFile;
	} # if (!defined($g_strConfFile))

	if (sysopen($fh, $file, O_RDONLY))
	{
		while (defined($line = readline($fh)))
		{
			$idx = index($line, "#");
			$idx = (-1 eq $idx) ? length($line) : $idx;
			$line = substr($line, 0, $idx);

			if (length($line) && $line =~ /\S+/)
			{
				if ($line =~ /(\S+)[\s]*=[\s]*(\S+)/)
				{
					# Save the name/value pair
					$g_Options{$1} = $2;
				}
				else
				{
					console_print("Invalid syntax on line $. of configuration file $file\n");
				} # if ($line =~ /(.+?)[\s]*=[\s]*(.+?)/)
			} # if (length($line) && $line =~ /\S+/)
		} # while (defined($line = readline($fh)))

		close($fh);
	} # if (sysopen($fh, $file, O_RDONLY))

	return(1);

} # parse_conf_file

#**********************************************************************
# Name:			get_conf_option
# Description:	Retrieves a parameter from the conf file
# Parameters:	Name of option.  Case sensitive.
#				Default value
# Return:		Option value
# Notes:
#**********************************************************************
sub get_conf_option($$)
{
	my ($name, $default) = @_;
	my $retval;

	$retval = $g_Options{$name};
	if (!defined($retval))
	{
		$retval = $default;
	} # if (!defined($retval))

	return($retval);

} # get_conf_option

#**********************************************************************
# Name:			log_request
# Description:	Logs the request XML that was sent to the ADMIN file system
# Parameters:	XML to log
# Return:		None.
# Notes:
#**********************************************************************
sub log_request($)
{
	my $xml = shift;

	if ($g_bDebug)
	{
		print("\n*** BEGIN REQUEST **************************************************\n");
		print($xml);
		print("\n*** END REQUEST ****************************************************\n");
	} # if ($g_bDebug)

} # log_request

#**********************************************************************
# Name:			log_response
# Description:	Logs the response XML that was received from the ADMIN file system
# Parameters:	XML to log
# Return:		None.
# Notes:
#**********************************************************************
sub log_response($)
{
	my $xml = shift;

	if ($g_bDebug)
	{
		print("\n*** BEGIN RESPONSE *************************************************\n");
		print($xml);
		print("\n*** END RESPONSE ***************************************************\n");
	} # if ($g_bDebug)

} # log_response

#**********************************************************************
# Name:			END
# Description:	Runs when the program ends
# Notes:
#**********************************************************************
END
{

} # END

1;

