#****************************************************************************
#
# Copyright (c) 2005 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Convert.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Convert;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("convert");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my ($self, $verb, $resource) = @_;

	# This command is only valid on a Linux box
	if ($^O ne "linux")
	{
		console_print("\nThis command is only valid on Linux.\n\n");
        return(1);
	} # if ($^O ne "linux")

	if (!defined($verb))
	{
		$self->Help();
		return(1);
	} # if (!defined($verb))

	$verb = lc($verb);
	if ("preview" eq $verb)
	{
		if (!defined($resource) || "" eq $resource)
		{
			$resource = "";
		} # if (!defined($resource) || "" eq $resource)

		$self->Preview($resource);
	}
	elsif ("commit" eq $verb)
	{
		$self->Commit();
	}
	else
	{
		$self->Help()
	}

	return(1);

} # Execute

#**********************************************************************
# Name:			Preview
# Description:	Handles the preview command
# Parameters:	Resource to preview
# Return:		None
# Notes:
#**********************************************************************
sub Preview($)
{
	my ($self, $resource) = @_;
	my $results;

	if( "" ne $resource )
	{
		$results = $self->ExecuteClstrLibRpc("[clstrlib.NWCLSTR_PreviewScriptTranslation,\\\"$resource\\\"]");
	}
	else
	{
		$results = $self->ExecuteClstrLibRpc("[clstrlib.NWCLSTR_PreviewAllScripts,\\\"$resource\\\"]");
	}

	# The results can come back as a string or an error number.  If it is an
	# error number, display a nicer message.  If it is a string, just
	# display it.
	if ($results =~ /^-?[\d]+$/)
	{
		if (-1 == $results)
		{
			console_print("\n\"$resource\" is not a valid cluster resource.\n\n");
		}
		elsif(-2 == $results)
		{
			console_print("\nThere was an LDAP error while previewing the script translations.\n\n");
		}
		else
		{
			console_print("\nAn unknown error occurred while previewing the script translations ($results).\n\n");
		}
	}
	elsif (length($results) == 0)
	{
		console_print("\nThe NCS Configuration Daemon did not return any data.\n\n");
	}
	else
	{
        console_print("\n$results\n");
	} # if ($results =~ /^-?[\d]+$/)

} # Preview

#**********************************************************************
# Name:			Commit
# Description:	Handles the commit command
# Parameters:	None
# Return:		None
# Notes:
#**********************************************************************
sub Commit()
{
	my ($self) = @_;
	my $results;

	$results = $self->ExecuteClstrLibRpc("[clstrlib.NWCLSTR_ConvertCluster]");

	# The results come back as an error code
	if (0 == $results)
	{
                system( "echo -n 'EXEC /opt/novell/ncs/bin/ncs-configd.py -init' > /proc/ncs/cluster" );
		console_print("\nThe cluster has successfully been converted.\n\n");
	}
	elsif(-1 == $results)
	{
		console_print("\nA cluster comprising of all NetWare nodes cannot be converted.\n");
		console_print("The cluster must first be converted to Linux, and all cluster\n");
		console_print("nodes must be running the Linux operating system.\n\n");
	}
	elsif(-2 == $results)
	{
		console_print("\nAn LDAP error occurred while converting the cluster.\n\n");
		console_print("The cluster was not converted.\n");
	}
	elsif(-255 == $results)
	{
		console_print("\nThe cluster has already been converted.\n\n");
	}
	else
	{
		console_print("\nAn unknown error occurred while converting the cluster ($results).\n");
		console_print("The cluster was not converted.\n\n");
	}

} # Commit


#**********************************************************************
# Name:			ExecuteClstrLibRpc
# Description:	Executes a RPC to the clstrlib module
# Parameters:	RPC to execute
# Return:		Return value from RPC
# Notes:
#**********************************************************************
sub ExecuteClstrLibRpc($)
{
	my ($self, $rpc) = @_;
	my $results;

	$results = `/opt/novell/ncs/bin/ncs-configd.py $rpc`;

	return($results)

} # ExecuteClstrLibRpc

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER CONVERT {PREVIEW | COMMIT}\n");
    console_print("  Finalizes the cluster conversion from NetWare to Linux after all nodes in\n");
	console_print("  a mixed cluster have been converted to Linux.  Specify a resource name with\n");
	console_print("  the Preview switch to view the resource load and unload script changes prior\n");
	console_print("  to finalizing the conversion.  Use the Commit switch without specifying a\n");
	console_print("  resource to finalize the conversion for all cluster resources.  The CLUSTER\n");
	console_print("  CONVERT command can only be executed on Linux cluster nodes.\n");
	console_print("Example: cluster convert preview res1\n");
    console_print("Example: cluster convert commit\n\n");
    
	return(1);

} # Help

1;

