#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Credentials.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Credentials;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;
use POSIX;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("credentials");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my ($self, $cluster) = @_;

	# The credentials are sent to the server in the clear, where they are then
	# encrypted.  Since they are sent in the clear, don't allow this command
	# to be ran on a remote host.
	#
	# ***************************************************************************
	#  !!!IT IS A SECURITY RISK TO REMOVE THIS CHECK.  DO SO AT YOUR OWN RISK!!!
	# ***************************************************************************
	if (is_remote())
	{
		console_print("\nThe Credentials command cannot be ran from a remote host.\n");
		console_print("The command must be ran from the server console.\n");
		return(1);
	} # if (is_remote())

	# We are on the local box where the credentials will not be
	# transmitted over the wire.  Go ahead with this command
	if ($cluster)
	{
		if (!is_netware())
		{
			# Non-NetWare
			GetCredentials(@_);			
		}
		else
		{
			# NetWare
			GetCredentialsNW(@_);
		} # if (!is_netware())
	}
	else
	{
		console_print("\nA cluster name must be specified.  For example: \n");
		console_print("cluster credentials A_Cluster\n");
	} # if ($cluster)

	return(1);

} # Execute

#**********************************************************************
# Name:			GetCredentials
# Description:	Retrieves the credentials on all systems except NetWare
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub GetCredentials(@)
{
	my ($self, $cluster) = @_;
	my $xml;
	my $blob;
	my $error;
	my $fh;
	my $host;  
	my %results;
	my $user;
	my $password;

	console_print("Enter the credentials for the specified cluster.  Press CTRL+C to cancel.\n");

	console_print("Username: ");
	$user = console_getstring();

	console_print("Password: ");
	$password = console_getpassword();

	console_print("Please wait...\n");

	# Encode the blob
	$blob = unpack("H*", "$user\0$password\0");
	
	# Format the request XML
	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>" .
		"<bccRequest><bccConfig><setCredentials>" .
		"<cluster>$cluster</cluster><blob>$blob</blob>" .
		"<clearText/>" .		# Tell the BCC to encrypt the credentials before they are stored
		"</setCredentials></bccConfig></bccRequest>";

	$user = "";
	$password = "";
	$blob = "";

	log_request($xml);
	
	# Write the command
	$host = $self->GetHost();
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Config.cmd");
	$self->WriteVfsFile($fh, $xml);

	# Get the result
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# Output the results
	$self->ParseBccResults($xml, \%results, "bccConfig", "setCredentials");
	$xml = "";
	
	if (0 == $results{"BccReplyCode"})
	{
		console_print("The credentials for cluster $cluster have been saved\n");		
	}
	else
	{
		console_print("There was an error saving the credentials for cluster $cluster\n");
		console_print("$results{\"BccReplyString\"} ($results{\"BccReplyCode\"})\n");
	} # if (0 == $results{"BccReplyCode"})
	
	return(1);

} # GetCredentials

#**********************************************************************
# Name:			GetCredentialsNW
# Description:	Retrieves the credentials on a NetWare system
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub GetCredentialsNW(@)
{
	my ($self, $cluster) = @_;
	my $xml;
	my $blob;
	my $error;
	my $fh;
	my $host;  
	my $user;
	my $password;
	my %results;

	# Parse the username and password if they exist.
	# The Perl function getopt() does not work here.
	for (my $idx = 0; $idx < $#ARGV + 1; $idx++)
	{
		if ($ARGV[$idx] eq "-u")
		{
			$user = $ARGV[$idx + 1];
			$idx++;
		} # if ($ARGV[$idx] eq "-u")
		
		if ($ARGV[$idx] eq "-p")
		{
			$password = $ARGV[$idx + 1];
			$idx++;
		} # if ($ARGV[$idx] eq "-u")
	} # for (my $idx = 0; $idx < $#ARGV + 1; $idx++)
	
	if (!defined($user) || !defined($password))
	{
		if (!defined($user) && !defined($password))
		{
			$xml = "<query>" .
						"<output>" .
							"<string>Enter the credentials for the specified cluster.  Press ESCAPE to cancel.\n</string>" .
							"<flag>-x</flag>" .		# Dummy flag
						"</output>" .						
						"<input>" .
							"<prompt>Username: </prompt>" .
							"<flag>-u</flag>" .
						"</input>" .
						"<password>" .
							"<prompt>Password: </prompt>" .
							"<flag>-p</flag>" .
						"</password>" .
					"</query>";

			console_print($xml);
		}
		else
		{
			console_print("There was an error retrieving the credentials from the NetWare system console.\n");
			console_print("If this error persists, please contact Novell Technical Support.\n");
		} # if (!defined($user) && !defined($password))
	}
	else
	{
		# Encode the blob
		$blob = unpack("H*", "$user\0$password\0");

		# Format the request XML
		$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>" .
			"<bccRequest><bccConfig><setCredentials>" .
			"<cluster>$cluster</cluster><blob>$blob</blob>" .
			"<clearText/>" .		# Tell the BCC to encrypt the credentials before they are stored
			"</setCredentials></bccConfig></bccRequest>";

		$user = "";
		$password = "";
		$blob = "";

		log_request($xml);

		# Write the command
		$host = $self->GetHost();
		$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Config.cmd");
		$self->WriteVfsFile($fh, $xml);

		# Get the result
		$self->ReadVfsFile($fh, $xml);
		$self->CloseVfsFile($fh);

		log_response($xml);

		# Output the results
		$self->ParseBccResults($xml, \%results, "bccConfig", "setCredentials");
		$xml = "";

		if (0 == $results{"BccReplyCode"})
		{
			console_print("\nThe credentials for cluster $cluster have been saved\n");		
		}
		else
		{
			console_print("There was an error saving the credentials for cluster $cluster\n");
			console_print("$results{\"BccReplyString\"} ($results{\"BccReplyCode\"})\n");
		} # if (0 == $results{"BccReplyCode"})
	} # if (!defined($user) || !defined($password))

	return(1);

} # GetCredentialsNW

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER CREDENTIALS {cluster}\n");
	console_print("  Sets the credentials on this cluster for the specified cluster in a\n");
	console_print("  Business Continuity Cluster.  The cluster you specify must have\n");
	console_print("  already been enabled for Business Continuity Clustering.\n");
	console_print("Example: cluster credentials A_Cluster\n");

	return(1);

} # Help

1;

