#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Join.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Join;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("join");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my $retVal = 0;

	if (!is_netware())
	{
		system( "echo 1 > /proc/sys/vm/drop_caches" );
		$retVal = DoJoin(@_);		
	}
	else
	{
		$retVal = DoJoinNW(@_);
	} # if (!is_netware())

	return($retVal);

} # Execute

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER JOIN\n");
	console_print("  Adds the node where the command is executed to the cluster and makes the\n");
	console_print("  node visible to other servers in the cluster.  Cluster Services software\n");
	console_print("  must already be installed on a node for it to join the cluster.\n");
	console_print("Example: cluster join\n");

	return(1);

} # Help

#**********************************************************************
# Name:			DoJoin
# Description:	Executes the join on Linux
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub DoJoin(@)
{
	my $self = shift;
	my $fh;
	my $host;
	my $xml;
	my $vfsfile	= "/Novell/Cluster/Node.cmd";
	my $retVal = 0;

	$host = $self->GetHost();

	if ($self->IsMember($host))
	{
		console_print("This node is already a member of the cluster.\n");
		return(1);		
	} # if ($self->IsMember($host))

    $xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	$xml .= "<ncsRequest><join/></ncsRequest>";

	log_request($xml);

	# Tell the cluster software to join the cluster
    $fh = $self->OpenVfsFile($host, $vfsfile);
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match because sometimes sysread returns
	# the XML twice.  Don't ask me why...
	if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
	{
		console_print("Joining...\n");
        # Wait while the join is in progress...
		while(1)
		{
			sleep(1);

			# The status verb is invalid.  However, NCS will return "busy" until the
			# join operation completes at which time it will return invalid XML...
			$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
			$xml .= "<ncsRequest><status/></ncsRequest>";

			log_request($xml);

			$fh = $self->OpenVfsFile($host, $vfsfile);
			$self->WriteVfsFile($fh, $xml);
			$self->ReadVfsFile($fh, $xml);
			$self->CloseVfsFile($fh);

			log_response($xml);

			if ($xml !~ /<ncsReply>[\s]*<failed>.*?already in progress<\/failed>[\s]*<\/ncsReply>/s)
			{
				last;
			} # if ($xml !~ /<ncsReply>[\s]*<failed>*already in progress<\/failed>[\s]*<\/ncsReply>/s)
		} # while (1)

		console_print("Now a member of\n");
		$self->ShowClusterView($host);
		$retVal = 1;
	}
	else
	{
		console_print("There was an error while joining the cluster\n");
		console_print("Error: \"$1\"\n");
	} # if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
	
	return($retVal);

} # DoJoin

#**********************************************************************
# Name:			DoJoinNW
# Description:	Executes the join on NetWare
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub DoJoinNW(@)
{
	my $self = shift;
	my $fh;
	my $host;
	my $xml;
	my $vfsfile	= "/Novell/Cluster/Node.cmd";
	my $joining = 0;

	# Parse for the "joining" flag
	# The Perl function getopt() does not work here.
	for (my $idx = 0; $idx < $#ARGV + 1; $idx++)
	{
		if ($ARGV[$idx] eq "-2")
		{
			$joining = 1;
			last;
		} # if ($ARGV[$idx] eq "-2")
	} # for (my $idx = 0; $idx < $#ARGV + 1; $idx++)

	$host = $self->GetHost();

	if ($joining)
	{
		# This is the second time through this handler.  Start the join
		# and wait for it to complete.

		$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
		$xml .= "<ncsRequest><join/></ncsRequest>";

		log_request($xml);

		# Tell the cluster software to join the cluster
        $fh = $self->OpenVfsFile($host, $vfsfile);
		$self->WriteVfsFile($fh, $xml);
		$self->ReadVfsFile($fh, $xml);
		$self->CloseVfsFile($fh);

		log_response($xml);

		# The grouping must be a minimal match because sometimes sysread returns
		# the XML twice.  Don't ask me why...
		if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
		{
			while(1)
			{
				sleep(1);
	
				# The status verb is invalid.  However, NCS will return "busy" until the
				# join operation completes at which time it will return invalid XML...
				$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
				$xml .= "<ncsRequest><status/></ncsRequest>";
	
				log_request($xml);
	
				$fh = $self->OpenVfsFile($host, $vfsfile);
				$self->WriteVfsFile($fh, $xml);
				$self->ReadVfsFile($fh, $xml);
				$self->CloseVfsFile($fh);
	
				log_response($xml);
	
				if ($xml !~ /<ncsReply>[\s]*<failed>.*?already in progress<\/failed>[\s]*<\/ncsReply>/s)
				{
					last;
				} # if ($xml !~ /<ncsReply>[\s]*<failed>*already in progress<\/failed>[\s]*<\/ncsReply>/s)
			} # while (1)
	
			console_print("Now a member of\n");
			$self->ShowClusterView($host);
		}
		else
		{
			console_print("There was an error while joining the cluster\n");
			console_print("Error: \"$1\"\n");
		} # if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
	}
	else
	{
		# Are we already a member?
		if (!$self->IsMember($host))
		{
			# Inform the user that we are joining.  The handler will be called again
			# to perform the actual join.
			$xml = "<query>" .
						"<output>" .
							"<string>Joining...\n</string>" .
							"<flag>-2</flag>" .
						"</output>" .
					"</query>";

			console_print($xml);
		}
		else
		{
			console_print("This node is already a member of the cluster.\n");
		} # if (!$self->IsMember($host))
	} # if ($joining)

	return(1);

} # DoJoinNW

1;

