#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Leave.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Leave;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("leave");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my $retVal = 0;

	if (!is_netware())
	{
		$retVal = DoLeave(@_);		
	}
	else
	{
		$retVal = DoLeaveNW(@_);
	} # if (!is_netware())

	return($retVal);

} # Execute

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER LEAVE\n");
	console_print("  Removes the node where the command is executed from the cluster.  The\n");
	console_print("  node will not be visible to other servers in the cluster.\n");
	console_print("Example: cluster leave\n\n");

	return(1);

} # Help

#**********************************************************************
# Name:			DoLeave
# Description:	Executes the leave command
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub DoLeave(@)
{
	my $self = shift;
	my $fh;
	my $host;
	my $xml;
	my $vfsfile	= "/Novell/Cluster/Node.cmd";
	my $retVal = 0;

	$host = $self->GetHost();

	if (!$self->IsMember($host))
	{
		console_print("This node is not a member of the cluster\n");
		return(1);		
	} # if (!$self->IsMember($host))

	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	$xml .= "<ncsRequest><leave/></ncsRequest>";

	log_request($xml);

	# Tell the cluster software to leave the cluster
    $fh = $self->OpenVfsFile($host, $vfsfile);
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
	{
		console_print("Leaving...\n");
		# Wait while the leave is in progress...
		while(1)
		{
			sleep(1);

			# The status verb is invalid.  However, NCS will return "busy" until the
			# leave operation completes at which time it will return invalid XML...
			$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
			$xml .= "<ncsRequest><status/></ncsRequest>";

			log_request($xml);

			$fh = $self->OpenVfsFile($host, $vfsfile);
			$self->WriteVfsFile($fh, $xml);
			$self->ReadVfsFile($fh, $xml);
			$self->CloseVfsFile($fh);

			log_response($xml);

			if ($xml !~ /<ncsReply>[\s]*<failed>.*?already in progress<\/failed>[\s]*<\/ncsReply>/s)
			{
				last;
			} # if ($xml !~ /<ncsReply>[\s]*<failed>*already in progress<\/failed>[\s]*<\/ncsReply>/s)
		} # while (1)

		# TODO: Should match NetWare
		# NetWare: "No longer a member of cluster <cluster name>"
		console_print("No longer a member of cluster\n");
		$retVal = 1;
	}
	else
	{
		console_print("There was an error while leaving the cluster\n");
		console_print("Error: \"$1\"\n");
	} # if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
	
	return($retVal);

} # DoLeave

#**********************************************************************
# Name:			DoLeaveNW
# Description:	Executes the leave command on NetWare
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub DoLeaveNW(@)
{
	my $self = shift;
	my $fh;
	my $host;
	my $xml;
	my $vfsfile	= "/Novell/Cluster/Node.cmd";
	my $leaving = 0;
	my $retVal = 0;

	# Parse for the "leaving" flag
	# The Perl function getopt() does not work here.
	for (my $idx = 0; $idx < $#ARGV + 1; $idx++)
	{
		if ($ARGV[$idx] eq "-2")
		{
			$leaving = 1;
			last;
		} # if ($ARGV[$idx] eq "-2")
	} # for (my $idx = 0; $idx < $#ARGV + 1; $idx++)

	$host = $self->GetHost();

	if ($leaving)
	{
		# This is the second time through this handler.  Start the leave
		# and wait for it to complete.

		$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
		$xml .= "<ncsRequest><leave/></ncsRequest>";

		log_request($xml);

		# Tell the cluster software to leave the cluster
		$fh = $self->OpenVfsFile($host, $vfsfile);
		$self->WriteVfsFile($fh, $xml);
		$self->ReadVfsFile($fh, $xml);
		$self->CloseVfsFile($fh);

		log_response($xml);

		# The grouping must be a minimal match because sometimes sysread returns
		# the XML twice.  Don't ask me why...
		if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
		{
			while(1)
			{
				sleep(1);
	
				# The status verb is invalid.  However, NCS will return "busy" until the
				# leave operation completes at which time it will return invalid XML...
				$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
				$xml .= "<ncsRequest><status/></ncsRequest>";
	
				log_request($xml);
	
				$fh = $self->OpenVfsFile($host, $vfsfile);
				$self->WriteVfsFile($fh, $xml);
				$self->ReadVfsFile($fh, $xml);
				$self->CloseVfsFile($fh);
	
				log_response($xml);
	
				if ($xml !~ /<ncsReply>[\s]*<failed>.*?already in progress<\/failed>[\s]*<\/ncsReply>/s)
				{
					last;
				} # if ($xml !~ /<ncsReply>[\s]*<failed>*already in progress<\/failed>[\s]*<\/ncsReply>/s)
			} # while (1)
	
			# TODO: Should match NetWare
			# NetWare: "No longer a member of cluster <cluster name>"
			console_print("No longer a member of cluster\n");
		}
		else
		{
			console_print("There was an error while leaving the cluster\n");
			console_print("Error: \"$1\"\n");
		} # if ($xml !~ /<ncsReply>[\s]*<failed>(.*?)<\/failed>[\s]*<\/ncsReply>/s)
	}
	else
	{
		# Are we a member?
		if ($self->IsMember($host))
		{
			# Inform the user that we are leaving.  The handler will be called again
			# to perform the actual leave.
			$xml = "<query>" .
						"<output>" .
							"<string>Leaving...\n</string>" .
							"<flag>-2</flag>" .
						"</output>" .
					"</query>";

			console_print($xml);
		}
		else
		{
			console_print("This node is not a member of the cluster.\n");
		} # if ($self->IsMember($host))
	} # if ($leaving)

	return(1);

} # DoLeaveNW

1;

