#****************************************************************************
#
# Copyright (c) 2004 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Maintenance.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Maintenance;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("maintenance");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my ($self, $mode, $duration) = @_;
	my $fh;
	my $host;
	my $xml;
	my $err;
	my $state;
	my $expire_epoch;
	my $output_string;
	my $date;
	my $input_epoch;
	my $current_epoch;
	my $vfsfile	= "/Novell/Cluster/ClusterManage.cmd";

	if ($mode)
	{
		$mode = lc($mode);
		if ($mode ne "on" && $mode ne "off")
		{
			$self->Help();
			return(0);
		} # if ($mode ne "on" && $mode ne "off")
	} # if ($mode)

	if($duration)
	{
		#Duration is allowed only if the cluster is homogeneous with SP3 and above
		$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
		$xml .= "<ncsRequest><clusterManage><InfoclusterVersion>";
		$xml .= "<version>OES2018SP3</version>";
		$xml .= "</InfoclusterVersion></clusterManage></ncsRequest>";

		log_request($xml);
		$host = $self->GetHost();
		$fh = $self->OpenVfsFile($host, $vfsfile);
		$self->WriteVfsFile($fh, $xml);
		$self->ReadVfsFile($fh, $xml);
		$self->CloseVfsFile($fh);

		log_response($xml);

		$xml =~ /<result value=\"(.*?)\">/s;
		$err = $1;
		if(defined($err) && 0 == $err)
		{
			$xml =~ /<SafeToUpgradeFS>(.*?)<\/SafeToUpgradeFS>/s;
			my $safe = $1;
			if($safe ne "yes")
			{
				console_print("All the nodes of the cluster are not SP3 and above.\n");
				console_print("Duration can be specified only if all nodes are SP3 and above.\n");
				return(2);
			}

		}
	}


	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	$xml .= "<ncsRequest><clusterManage><controlCluster><action>";

	if ($mode)
	{
		$xml .= "<maintenance state=\"$mode\"/>";
	}
	else
	{
		$xml .= "<maintenance/>";
	}

	$xml .= "</action>";

	if($duration)
	{
		$input_epoch = `date +%s --date=\"$duration\"`;
		my $ret = $?;
		if($ret)
		{
			print("Please specify a timestamp that is valid with the date command.\n");
			return(3);
		}
		$current_epoch = time();
		if($input_epoch <= $current_epoch)
		{
			console_print("Please specify a time greater than the current time.\n");
			return(4);
		}
		$xml .= "<duration>$input_epoch</duration>";
	}# if ($duration)

	$xml .= "</controlCluster></clusterManage></ncsRequest>";

	log_request($xml);

	# Output the standard NCS information
	$host = $self->GetHost();
   $fh = $self->OpenVfsFile($host, $vfsfile);
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# The grouping must be a minimal match
   $xml =~ /<result value=\"(.*?)\">/s;
	$err = $1;

	if (defined($err) && 0 == $err)
	{
		# Success
		$xml =~ /<maintenance state=\"(.*?)\"\/>/s;
		$state = $1;
		if($xml =~ /<epoch value=\"(.*?)\"\/>/s)
		{
			$expire_epoch = $1;
		}

		if ($state)
		{
			if ($state eq "on")
			{
				if ($mode)
				{
					console_print("Maintenance command issued to all the nodes.\n");
				}
				elsif(not $expire_epoch)
				{
					console_print("The cluster is in maintenance mode.\n");
				}
				else
				{
					$date = localtime($expire_epoch);
					console_print("The cluster is in maintenance mode till $date.\n");
				}
			}
			elsif ( $state eq "off")
			{
				if ($mode)
				{
					console_print("Maintenance command issued to all the nodes.\n");
				}
				else
				{
					console_print("The cluster is not in maintenance mode.\n");
				}
			} # if ($xml eq "on")
			elsif( $state eq "nochange")
			{
				if($mode eq "on")
				{
					console_print("The cluster is already in maintenance mode.\n");
				}
				else
				{
					console_print("Maintenance mode is already turned off.\n");
				}
			}
		}
		else
		{
			console_print("There was an error retrieving the maintenance state.\n");
		} # if ($xml)
	#} # if ($mode)
	}
	else
	{
      console_print("There was an error issuing the Maintenance command ($err).\n");
	} # if (defined($err) && 0 == $err)

	return(1);

} # Execute

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER MAINTENANCE {ON / OFF} {TIMESTAMP}\n");
	console_print("  Set or reset cluster maintenance mode.  Running this command without\n");
	console_print("  the {ON / OFF} parameter will report on the maintenance status of the\n");
	console_print("  cluster (e.g. if cluster maintenance mode is on or off).\n");
	console_print("  If the timestamp is specified, the maintenance mode is automatically\n");
	console_print("  turned off after the time gets expired.\n");
	console_print("Example: cluster maintenance on '2021-04-01 13:50'\n");

	return(1);

} # Help

1;

