#****************************************************************************
#
# Copyright (c) 2003 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Migrate.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Migrate;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;
use XML::LibXML;
require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("migrate");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my $self = shift;
	my $retval;

	if (!$_[0])
	{
		$self->Help();
		return(0);
	} # if (!$_[0])

	if ($_[0] =~ /\//)
	{
		# Make sure the command line is valid

		# Validate the source cluster first.  Based on the check above $_[0] is defined.
		if ($_[0] !~ /.+\/.+/)
		{
			$self->Help();
			return(0);
		} # if ($_[0] !~ /.+\/.+/)

		# The source is OK.  Validate the destination cluster next
		if (!defined($_[1]) || ($_[1] !~ /.+\/.+/))
		{
			$self->Help();
			return(0);
		} # if (!defined($_[1]) || ($_[1] !~ /.+\/.+/))

		# The command line is validated.  Execute the BCC migrate command.
		$retval = $self->ExecuteBccCommand(@_);
	}
	else
	{
		$retval = $self->ExecuteNcsCommand(@_);
	} # if ($_[0] =~ /\//)

	return($retval);

} # Execute

#**********************************************************************
# Name:			ExecuteBccCommand
# Description:	Executes the BCC command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ExecuteBccCommand(@)
{
	my $self = shift;
	my %params;
	my $xml;
	my $host;
	my $fh;
	my $prompt;

	# Get the cluster parameters
	$_[0] =~ /(^.*)\/(.*$)/;
	$params{"SourceCluster"} = $1;
	$params{"Resource"} = $2;

	$_[1] =~ /(^.*)\/(.*$)/;
	$params{"DestinationCluster"} = $1;
	$params{"DestinationNode"} = $2;

	if (is_prompt_set() && !$self->CheckClusterConnections())
	{
		if (is_netware())
		{
			# Parse for the prompt
			for (my $idx = 0; $idx < $#ARGV + 1; $idx++)
			{
				if ($ARGV[$idx] eq "-q")
				{
					$prompt = $ARGV[$idx + 1];
					last;
				} # if ($ARGV[$idx] eq "-q")
			} # for (my $idx = 0; $idx < $#ARGV + 1; $idx++)

			if (!defined($prompt))
			{
				# Prompt the user if the operation is OK
				$xml =
					"<query>".
						"<input>".
							"<prompt>".
								"\nThis cluster is currently experiencing connection problems.\n".
								"Are you sure you want to continue with the resource migration (Y/N)? ".
							"</prompt>".
							"<flag>-q</flag>".
						"</input>".
					"</query>";

				console_print($xml);
				return(0);
			} # if (!defined($prompt))
		}
		else
		{
			console_print(
				"\nThis cluster is currently experiencing connection problems.\n".
				"Are you sure you want to continue with the resource migration (Y/N)? ");
			$prompt = console_getstring();
		} # if (is_netware())

		# See what the user wants to do
		if (uc(substr($prompt, 0, 1)) ne "Y")
		{
			# The user cancelled the operation
			console_print("\nResource migration cancelled.\n");
			return(0);
		} # if (uc(substr($prompt, 0, 1)) ne "Y")
	} # if (is_prompt_set() && !$self->CheckClusterConnections())

	# Format the output XML that gets written to the VFS file
	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>" .
		"<bccRequest><bccManage><migrate><name>$params{\"Resource\"}</name>" .
		"<targetNode>$params{\"DestinationNode\"}</targetNode>" .
      "<sourceCluster>$params{\"SourceCluster\"}</sourceCluster>" .
		"<targetCluster>$params{\"DestinationCluster\"}</targetCluster>" .
		"</migrate></bccManage></bccRequest>";

	log_request($xml);

	$host = $self->GetHost();

	if (!$self->IsBCCLoaded($host))
	{
		console_print("\nERROR: The Novell Business Continuity Cluster is not running.\n");
		console_print("You cannot perform a Business Continuity Cluster migrate.\n\n");
		return(1);
	} # if (!$self->IsBCCLoaded($host))

	if ("*" eq $params{"Resource"})
	{
		console_print("\nMigrating all resources.  Please wait...\n\n");
	}
	else
	{
		console_print("\nMigrating resource $params{\"Resource\"}.  Please wait...\n\n");
	} # if ("*" eq $params{"Resource"})

	# Write the command
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Manage.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# Output the results
	$self->ParseBccResults($xml, \%params, "bccManage", "migrate");
	if (0 == $params{"BccReplyCode"})
	{
		console_print("Resource migration has been scheduled successfully\n");
	}
	else
	{
		console_print("$params{\"BccReplyString\"} ($params{\"BccReplyCode\"})\n");
	} # if (0 == $params{"bccResponseCode"})

	return(1);

} # ExecuteBccCommand

#**********************************************************************
# Name:			ExecuteNcsCommand
# Description:	Executes the NCS command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub ExecuteNcsCommand(@)
{
	my ($self, $resource, $node) = @_;
	my $fh;
	my $host;
	my $xml;
	my %results;

	if (!$resource || !$node)
	{
		$self->Help();
		return(0);
	} # if (!$resource || !$node)
	if ("$node" eq ".")
	{
		$node = $self->GetCurrentNode();
	}
	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	$xml .= "<ncsRequest><clusterManage><controlResource>";
	$xml .= "<name>$resource</name><action><migrate/></action>";
	$xml .= "<nodeName>$node</nodeName>";
	$xml .= "</controlResource></clusterManage></ncsRequest>";

	log_request($xml);

	$host = $self->GetHost();
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/ClusterManage.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	%results = $self->ParseNcsResults($xml);

	if (512 == $results{"code"})
	{
		console_print("\nResource $resource does not exist\n", 0);
	}
	elsif (509 == $results{"code"})
	{
		console_print("\nNode $node does not exist\n", 0);
	}
	else
	{
		if( $xml =~ /<preferredNodes>(.*?)<\/preferredNodes>/s )
		{
			console_print( "\nPreferred nodes for resource '$resource'\n" );
			$xml = $1;
                        while( $xml =~ /<node>(.*?)<\/node>/g )
			{
				console_print( "  $1\n" );
			}
		}
		# Output the state of the resource
		$self->ShowResourceList($host, 0, $resource);		# 0 = Don't show header
	}

	return(1);

} # ExecuteNcsCommand

#**********************************************************************
# Name:			ParseNcsResults
# Description:	Parses the results from an NCS command
# Parameters:	Buffer to parse
# Return:		Hash with results
#					code -> Error code
#					description -> Description string
# Notes:
#**********************************************************************

# TODO: This should be moved to the base class

sub ParseNcsResults($)
{
	my ($self, $xml) = @_;
	my %results;

	# TODO: Replace 506 with a meaningful constant
	$results{"code"} = 506;									# NCS_UNKNOWN
	$results{"description"} = "Error parsing results";

	if ($xml =~/<ncsReply>(.*?)<\/ncsReply>/s)
	{
		$xml = $1;
		if ($xml =~/<result[\s]+value=\"([\d]+)\">(.*?)<\/result>/s)
		{
			$results{"code"} = $1;

			$xml = $2;
			if ($xml =~/<description>(.*?)<\/description>/s)
			{
				$results{"description"} = $1;
			} # if ($xml =~/<description>(.*?)<\/description>/s)
		} # if ($xml =~/<result[\s]+value=\"([\d]+)\">(.*?)<\/result>/s)
	} # if ($xml =~/<ncsReply>(.*?)<\/ncsReply>/s)

	return(%results);

} # ParseNcsResults

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nSTANDARD CLUSTER MIGRATE:\n");
	console_print("CLUSTER MIGRATE {resource} {node name|-list|-most|-next}\n");
        console_print("  node name  Migrates the specified resource from the node where it is currently\n");
	console_print("  running to the node you specify in the command.  The node you migrate the\n");
	console_print("  resource to must be running in the cluster and also be in the resource's\n");
	console_print("  assigned nodes list.\n");
        console_print("  -l, -list  Shows the preferred nodes list for the specified resource.\n");
        console_print("  -m, -most  Migrates the specified resource to the most preferred node\n");
        console_print("  currently in the cluster. If the resource is running on such a node, no\n" );
        console_print("  action is taken.\n");
        console_print("  -n, -next  Migrates the resource to the node in the preferred node list\n" );
        console_print("  that is next in order to the node where the resource is currently running.\n");
        console_print("  If there is no next node or all such nodes are not in the cluster currently,\n" );
        console_print("  it searches from the beginning of the resource's preferred nodes list.\n");
        console_print("  No action is taken if a new destination could not be determined after the\n");
        console_print("  search is exhausted.\n");
        console_print("Example: cluster migrate res1 -list\n");
        console_print("Example: cluster migrate res1 node2\n\n");
    console_print("BUSINESS CONTINUITY CLUSTER MIGRATE*:\n");
	console_print("CLUSTER MIGRATE [--noprompt] {source}/{resource} {destination}/{node name}\n");
	console_print("  Migrates the specified resource from the given source cluster to the\n");
	console_print("  given target cluster.  Specifying '*' for the resource name will\n");
	console_print("  migrate all BCC enabled resources.  Specifying '*' for the node name\n");
	console_print("  will online the resource at the most preferred node.\n\n");
	console_print("  If the current cluster is experiencing connection problems, the user\n");
	console_print("  will be prompted to continue the migration.  The optional '--noprompt'\n");
	console_print("  flag can be used to ignore the connection status, and should be used when\n");
	console_print("  scripting resource migrations.\n");
	console_print("Example: cluster migrate cluster_1/res1 cluster_2/*\n");
	console_print("* NOTE: A Novell Business Continuity Cluster must be installed.\n\n");

	return(1);

} # Help

1;

