#****************************************************************************
#
# Copyright (c) 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Nsmi.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Nsmi;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:		Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:	Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("nsmi");

} # Param

#**********************************************************************
# Name:		Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:	true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my $self = shift;

	my $cluster_dn;
	my $fh;
	my $host;
	my %load_scripts;
	my %resources;
	my %results;
	my %unload_scripts;
	my $xml;

	# Variables to hold options
	my $continue;
	my $resource_name;
	my $script_id;
	my $script_type;

	# Make sure BCC is loaded
	$host = $self->GetHost();
	if (!$self->IsBCCLoaded($host))
	{
		$self->Help();
		return(0);
	} # if (!$self->IsBCCLoaded($host))

	# Command must be ran directly on a node
	if (is_remote())
	{
		console_print(
			"This command cannot be executed from a remote host.  Please ".
			"execute this directly on a cluster node.\n");
		return(0);
	} # if ($is_remote())

	# Make sure node is a member of a cluster.
	if (!$self->IsMember($host))
	{
		console_print(
			"This node is not a member of a cluster.  Please join ".
			"the node to a cluster before continuing.\n");
		return(0);
	} # if (!$self->IsMember($host))

	# Make sure bccutil is present.
	if (!(-f "sys:\\system\\bccutil.nlm" || -f "/opt/novell/bcc/sbin/bccutil"))
	{
		console_print("The required software to execute this command is not installed.\n");
		throw();
	} # if (!(-f "sys:\\system\\bccutil.nlm" ...
	
	# Get the cluster DN.
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/ClusterConfig.xml");
	$self->ReadVfsFile($fh, $xml);

	if ($xml =~ /<distinguishedName>(.*)<\/distinguishedName>/)
	{
		$cluster_dn = $1;

		if (is_netware())
		{
			# Convert to LDAP format
			$cluster_dn =~ s/\./,/g;
			$cluster_dn =~ s/,T=(.*)$//g;
			$cluster_dn =~ s/^,//g;
		} # if (is_netware())
	}
	else
	{
		console_print("Could not determine the distinguished name of the cluster.\n");
		throw("");
	} # if ($xml =~ /<distinguishedName>(.*)<\/distinguishedName>/)

	# Parse for options
	for (my $idx = 0; $idx < $#ARGV + 1; $idx++)
	{
		if ($ARGV[$idx] eq "--resource")
		{
			$resource_name = uc($ARGV[$idx + 1]);
		}
		elsif ($ARGV[$idx] eq "--continue")
		{
			$continue = uc($ARGV[$idx + 1]);
		}
		elsif ($ARGV[$idx] eq "--type")
		{
			$script_type = uc($ARGV[$idx + 1]);
		}
		elsif ($ARGV[$idx] eq "--id")
		{
			$script_id = uc($ARGV[$idx + 1]);
		} # if ($ARGV[$idx] eq "...
	} # for (my $idx = 0; $idx < $#ARGV + 1; $idx++)

	# Get the list of resources.
	# Format the output XML that gets written to the VFS file
	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>".
		"<bccRequest><bccManage><getResourceStatus filter=\"bcc\">".
	        "<cluster>.</cluster>".
        "</getResourceStatus></bccManage></bccRequest>";

	log_request($xml);

	# Write the command
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Manage.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# Check the results
	$self->ParseBccResults($xml, \%results, "bccManage", "getResourceStatus");
	if (0 != $results{"BccReplyCode"})
	{
		console_print("Could not get the resource list from the cluster.\n");
		console_print("\n$results{\"BccReplyString\"} ($results{\"BccReplyCode\"})\n");
		throw("");
	} # if (0 != $results{"BccReplyCode"})

	# Build resource status hash
	while ($xml =~ m/<name>(.*)<\/name>/g)
	{
		my $name = uc($1);
		$xml =~ m/<state>(.*)<\/state>/g;
		$resources{$name} = uc($1);
	} # while ($xml =~ m/<name>(.*)<\/name>/g)

	if (!%resources)
	{
		console_print("\nNo BCC-enabled cluster resources found.\n");
		return(1);
	} # if (0 == keys(%resources))

	my $resource_list_text = "\nBCC-Enabled Cluster Resources:\n";

	foreach my $key (sort(keys(%resources)))
	{
		$resource_list_text .= "  $key\n";
	} # foreach my $key (sort(keys(%resources)))

	while (!defined($resource_name) || !defined($resources{$resource_name}))
	{
		# Prompt for resource name
		if (is_netware())
		{
			$xml =
				"<query>".
					"<input>".
						"<prompt>".
							$resource_list_text.
							"Please enter the name of a resource (press ESC to cancel):".
						"</prompt>".
						"<flag>--resource</flag>".
					"</input>".
				"</query>";

			console_print($xml);
			return(0);
		}
		else
		{
			console_print($resource_list_text);
			console_print("Please enter the name of a resource (press CTRL-C to cancel): ");
			$resource_name = uc(console_getstring());
		} # if (is_netware())
	} # while (!defined($resource_name) || !defined($resources{$resource_name}))

	if ($resources{$resource_name} !~ m/^OFFLINE|SECONDARY$/)
	{
		console_print("Resource '$resource_name' must be in an offline state.\n");
		console_print("Please make sure '$resource_name' is offline before continuing.\n");
		return(0);
	} # if ($resources{$resource_name} !~ m/^OFFLINE|SECONDARY$/)

	# Get the resource information from the cluster
	# Format the output XML that gets written to the VFS file
	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>".
		"<bccRequest><bccConfig><getResourceInfo>".
	        "<name>$resource_name</name>".
        "</getResourceInfo></bccConfig></bccRequest>";

	log_request($xml);

	# Write the command
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Config.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# Check the results
	$self->ParseBccResults($xml, \%results, "bccConfig", "getResourceInfo");
	if (0 != $results{"BccReplyCode"})
	{
		console_print("Could not get the resource information from the cluster.\n");
		console_print("\n$results{\"BccReplyString\"} ($results{\"BccReplyCode\"})\n");
		throw("");
	} # if (0 != $results{"BccReplyCode"})

	# Parse load scripts
	if ($xml =~ m/<loadSettings>(.*)<\/loadSettings>/s)
	{
		my $load_settings_xml = $1;
	
		while ($load_settings_xml =~ m/<name>(.*?)<\/name>.*?<order>(.*?)<\/order>/sg)
		{
			$load_scripts{$2} = $1;
		} # while ($load_settings_xml =~ m/<name>(.*?)<\/name>.*?<order>(.*?)<\/order>/sg)
	} # if ($xml =~ m/<loadSettings>(.*)<\/loadSettings>/s)

	# Parse unload scripts
	if ($xml =~ m/<unloadSettings>(.*)<\/unloadSettings>/s)
	{
		my $unload_settings_xml = $1;
	
		while ($unload_settings_xml =~ m/<name>(.*?)<\/name>.*?<order>(.*?)<\/order>/sg)
		{
			$unload_scripts{$2} = $1;
		} # while ($unload_settings_xml =~ ...
	} # while ($unload_settings_xml =~

	while (!defined($script_type) || $script_type !~ m/^LOAD|UNLOAD/)
	{
		# Prompt for script type
		if (is_netware())
		{
			$xml =
				"<query>".
					"<input>".
						"<prompt>".
							"\nPlease enter the type of business continuity script, either LOAD or UNLOAD (press ESC to cancel):\n".
						"</prompt>".
						"<flag>--type</flag>".
					"</input>".
				"</query>";

			console_print($xml);
			return(0);
		}
		else
		{
			console_print("\nPlease enter the type of business continuity script, either LOAD or UNLOAD (press CTRL-C to cancel):\n");
			$script_type = uc(console_getstring());
		} # if (is_netware())
	} # while (!defined($script_type) || $script_type !~ m/^LOAD|UNLOAD/))

	my $script_prompt_text;

	if ("LOAD" eq $script_type)
	{
		if (!%load_scripts)
		{
			console_print("There are no business continuity load scripts for resource $resource_name.\n");
			return(1);
		} # if (!%load_scripts)

		$script_prompt_text = "\nBusiness continuity load scripts for resource $resource_name:\n";
		$script_prompt_text .=  "  ID\tSCRIPT NAME\n";
		foreach my $key (sort(keys(%load_scripts)))
		{
			$script_prompt_text .=  "  $key\t$load_scripts{$key}\n";
		} # foreach my $key (sort(keys(%load_scripts)))
	}
	else
	{
		if (!%unload_scripts)
		{
			console_print("There are no business continuity unload scripts for resource $resource_name.\n");
			return(1);
		} # if (!%unload_scripts)

		$script_prompt_text = "\nBusiness continuity unload scripts for resource $resource_name:\n";
		$script_prompt_text .=  "  ID\tSCRIPT NAME\n";
		foreach my $key (sort(keys(%unload_scripts)))
		{
			$script_prompt_text .=  "  $key\t$unload_scripts{$key}\n";
		} # foreach my $key (sort(keys(%unload_scripts)))
	} # if ("LOAD" eq $script_type)

	while (
		!defined($script_id)
			||
		("LOAD" eq $script_type && !defined($load_scripts{$script_id}))
			||
		("UNLOAD" eq $script_type && !defined($unload_scripts{$script_id})))
	{
		# Prompt for script ID
		if (is_netware())
		{
			$xml =
				"<query>".
					"<input>".
						"<prompt>".
							$script_prompt_text.
							"Please enter the id of the script to execute (press ESC to cancel):".
						"</prompt>".
						"<flag>--id</flag>".
					"</input>".
				"</query>";

			console_print($xml);
			return(0);
		}
		else
		{
			console_print($script_prompt_text);
			console_print("Please enter the id of the script to execute (press CTRL-C to cancel):");
			$script_id = console_getstring();
		} # if (is_netware())
	} #while (...

	if (!defined($continue))
	{
		# Prompt to continue
		if (is_netware())
		{
			$xml =
				"<query>".
					"<input>".
						"<prompt>".
							"\nPlease make sure resource $resource_name is offline in all clusters before continuing.  ".
							"Do you wish to proceed (Yes|No):".
						"</prompt>".
						"<flag>--continue</flag>".
					"</input>".
				"</query>";

			console_print($xml);
			return(0);
		}
		else
		{
			console_print("\nPlease make sure resource $resource_name is offline in all clusters before continuing.  ");
			console_print("Do you wish to proceed (Yes|No):");
			$continue = uc(console_getstring());
		} # if (is_netware())
	} # if (!defined($continue))

	if ("YES" ne $continue)
	{
		console_print("\nAborting...\n");
		return(1);
	} # if ("YES" ne $continue)

	my $bccutil_path = is_netware() ? "sys:\\system\\bccutil.nlm" : "/opt/novell/bcc/sbin/bccutil";

	# Execute the script
	my $command = "$bccutil_path nsmi -e -c $cluster_dn -r $resource_name";
	$command .= "LOAD" eq $script_type ? " -l" : " -u";
	$command .= " -i $script_id";
	system($command);

	console_print("\nScript executed.\nPlease check the system log for details.\n\n");
	
	return(1);
} # Execute

#**********************************************************************
# Name:		Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:	true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER NSMI\n");
	console_print("  BCC: Execute a single business continuity resource load or unload script.\n");
	console_print("  This command is to be used only for testing business continutiy scripts.\n");
	console_print("  Please make sure the appropriate resource is offline in all clusters\n");
	console_print("  in the BCC before executing this command.\n");
	console_print("Example: cluster nsmi\n");

	return(1);

} # Help

1;

