#****************************************************************************
#
# Copyright (c) 2006 - 2007 Novell, Inc. All Rights Reserved.
# THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
# NO PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED COPIED, DISTRIBUTED,
# REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, EXPANDED, COLLECTED,
# COMPILED, LINKED, RECAST, TRANSFORMED OR ADAPTED WITHOUT THE PRIOR WRITTEN
# CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION OF THIS WORK WITHOUT
# AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO CRIMINAL AND CIVIL LIABILITY.
#
#  Author:		Novell, Inc.
#  Workfile:	ClusterCliSnapin_Refresh.pm
#  Contents:	Command line implementation for Novell Cluster Services
#				and Novell Business Continuity Cluster.
#
#****************************************************************************

# The package name must be the same as the file name (including case)
package ClusterCliSnapin_Refresh;

#**********************************************************************
#	Use Statements
#**********************************************************************
use strict;
use warnings;
use diagnostics;
use ClusterCliUtils;
use Getopt::Long;

require ClusterCliSnapinInterface;
our @ISA = ("ClusterCliSnapinInterface");

#**********************************************************************
#	Functions
#**********************************************************************

#**********************************************************************
# Name:			Param
# Description:	Returns the name of the parameter this class handles
# Parameters:	None.
# Return:		Paramter string
# Notes:
#**********************************************************************
sub Param()
{
	return("refresh");

} # Param

#**********************************************************************
# Name:			Execute
# Description:	Executes the command handler
# Parameters:	List of parameters for the command.
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Execute(@)
{
	my $self = shift;
	my $cluster;
	my $resource;
	my $fh;
	my $host;
	my $xml;
	my %params;
    my %opts = ('p:i' => \$cluster,		# -p ==> Specifies that the cluster should be refreshed
				'r:s' => \$resource);	# -r ==> Specifies that a single resource or all resources should be refreshed

	GetOptions(%opts);

	# Format the XML
	$xml = "<virtualIO><datastream name=\"command\"/></virtualIO>";
	$xml .= "<bccRequest><bccManage><refresh>";

	if (!defined($cluster) && !defined($resource))
	{
		Help();
		return(1);		
	} # if (!defined($cluster) && !defined($resource))

	if (defined($cluster))
	{
		if ($cluster)
		{
			$xml .= "<cluster><revision>$cluster</revision></cluster>";			
		}
		else
		{
			$xml .= "<cluster/>";			
		} # if ($cluster)		
	} # if (defined($cluster))

	if (defined($resource))
	{
		# The -r switch was passed.  Determine if a resource was specified.
		if ("" ne $resource)
		{
			# A resource was passed in.  Get the name and optionally the revision.
			# The BCC engine uses a dot as a separator.  Try it first.
			if ($resource =~ /(^.+)\.(\d+$)/)
			{
				# Refresh a specific resource to a specific revision
				$xml .= "<resource>";
				$xml .= "<name>$1</name>";
				$xml .= "<revision>$2</revision>";
				$xml .= "</resource>";				
			}
			else
			{
				# Try the optional user friendly format
				if ($resource =~ /(^.+)=(\d+$)/)
				{
					# Refresh a specific resource to a specific revision
					$xml .= "<resource>";
					$xml .= "<name>$1</name>";
					$xml .= "<revision>$2</revision>";
					$xml .= "</resource>";				
				}
				else
				{
					# Refresh a specific resource (the revision does not matter)
					$xml .= "<resource>";
					$xml .= "<name>$resource</name>";
					$xml .= "<revision>0</revision>";
					$xml .= "</resource>";				
				} # if ($resource =~ /(^.+)=(\d+$)/)			
			} # if ($resource =~ /(^.+)=(\d+$)/)			
		}
		else
		{
			# Refresh all resources
			$xml .= "<resource/>";
		} # if ("" ne $resource)
	} # if (defined($resource))

	$xml .= "</refresh></bccManage></bccRequest>";

	log_request($xml);

	# Write the command
	$host = $self->GetHost();
	$fh = $self->OpenVfsFile($host, "/Novell/Cluster/BCC_Manage.cmd");
	$self->WriteVfsFile($fh, $xml);
	$self->ReadVfsFile($fh, $xml);
	$self->CloseVfsFile($fh);

	log_response($xml);

	# Output the results
	$self->ParseBccResults($xml, \%params, "bccManage", "refresh");
	if (0 == $params{"BccReplyCode"})
	{
		console_print("BCC: The local cache has been refreshed\n");
	}
	else
	{
		console_print("BCC: $params{\"BccReplyString\"} ($params{\"BccReplyCode\"})\n");
	} # if (0 == $params{"bccResponseCode"})

	return(1);

} # Execute

#**********************************************************************
# Name:			Help
# Description:	Prints help for the command handler
# Parameters:	List of parameters passed in to the help command
# Return:		true on success.  false otherwise.
# Notes:
#**********************************************************************
sub Help(@)
{
	console_print("\nCLUSTER REFRESH\n");
	console_print("  Used to refresh the internal BCC cache buffers on this node in this\n");
	console_print("  cluster.  Note that the other nodes in this cluster are not refreshed.\n");
	console_print("  The following are valid parameters to the refresh command:\n");
	console_print("  -p: [revision]: Refresh the cluster peer information where [revision]\n");
	console_print("      is an optional revision.  The resources are not refreshed unless\n");
	console_print("      explicitly specified.\n");
	console_print("  -r: [resource[=revision]]: Refresh the resource information where [resource]\n");
	console_print("      is an optional resource name to refresh.  If the resource name is\n");
	console_print("      not specified then all resources are refreshed.  If a resource\n");
	console_print("      name is specified then an optional revision can be specified by\n");
	console_print("      appending the revision number after the resource name.\n");
	console_print("Example: cluster refresh -p -r DATA_SERVER=5\n");

	return(1);

} # Help

1;

